@@ -1,8 +1,6 @@
 __all__ = [
     "eval_monte_carlo",
     "ev_monte_carlo",
-    "eval_lhs",
-    "ev_lhs",
     "eval_sinews",
     "ev_sinews",
     "eval_hybrid",
@@ -20,7 +18,6 @@ import grama as gr
 from grama import pipe, custom_formatwarning
 from scipy.stats import norm, lognorm
 from toolz import curry
-from pyDOE import lhs
 from numpy.linalg import cholesky, inv
 from numbers import Integral
 
@@ -30,7 +27,7 @@ warnings.formatwarning = custom_formatwarning
 # --------------------------------------------------
 @curry
 def eval_monte_carlo(model, n=1, df_det=None, seed=None, append=True, skip=False):
-    """Monte Carlo evaluation
+    r"""Monte Carlo evaluation
 
     Evaluates a given model at a given dataframe. Generates outer product
     with deterministic samples.
@@ -42,7 +39,7 @@ def eval_monte_carlo(model, n=1, df_det=None, seed=None, append=True, skip=False
             for nominal deterministic levels.
         seed (int): random seed to use
         append (bool): Append results to random values?
-        skip (bool): Skip evaluation?
+        skip (bool): Skip evaluation of the functions?
 
     Returns:
         DataFrame: Results of evaluation or unevaluated design
@@ -87,7 +84,7 @@ def eval_monte_carlo(model, n=1, df_det=None, seed=None, append=True, skip=False
             warnings.simplefilter("ignore")
             df_samp._plot_info = {
                 "type": "monte_carlo_inputs",
-                "var_rand": model.var_rand,
+                "var": model.var_rand,
             }
 
         return df_samp
@@ -107,64 +104,6 @@ def ev_monte_carlo(*args, **kwargs):
     return eval_monte_carlo(*args, **kwargs)
 
 
-## Latin Hypercube Sampling (LHS)
-# --------------------------------------------------
-@curry
-def eval_lhs(
-    model, n=1, df_det=None, seed=None, append=True, skip=False, criterion=None
-):
-    """Latin Hypercube evaluation
-
-    Evaluates a given model on a latin hypercube sample (LHS) using the model's
-    density.
-
-    Args:
-        model (gr.Model): Model to evaluate
-        n (numeric): Number of LHS samples to draw
-        df_det (DataFrame): Deterministic levels for evaluation; use "nom"
-            for nominal deterministic levels.
-        seed (int): Random seed to use
-        append (bool): Append results to conservative inputs?
-        skip (bool): Skip evaluation?
-        criterion (str): flag for LHS sample criterion
-            allowable values: None, "center" ("c"), "maxmin" ("m"),
-            "centermaxmin" ("cm"), "correlation" ("corr")
-
-    Returns:
-        DataFrame: Results of evaluation or unevaluated design
-
-    Notes:
-        - Wrapper on pyDOE.lhs
-
-    """
-    ## Set seed only if given
-    if seed is not None:
-        set_seed(seed)
-
-    ## Ensure sample count is int
-    if not isinstance(n, Integral):
-        print("eval_lhs() is rounding n...")
-        n = int(n)
-
-    ## Draw samples
-    df_quant = DataFrame(data=lhs(model.n_var_rand, samples=n), columns=model.var_rand)
-
-    ## Convert samples to desired marginals
-    df_rand = model.density.pr2sample(df_quant)
-    ## Construct outer-product DOE
-    df_samp = model.var_outer(df_rand, df_det=df_det)
-
-    if skip:
-        return df_samp
-    else:
-        return gr.eval_df(model, df=df_samp, append=append)
-
-
-@pipe
-def ev_lhs(*args, **kwargs):
-    return eval_lhs(*args, **kwargs)
-
-
 ## Marginal sweeps with random origins
 # --------------------------------------------------
 @curry
@@ -179,7 +118,7 @@ def eval_sinews(
     append=True,
     skip=False,
 ):
-    """Sweep study
+    r"""Sweep study
 
     Perform coordinate sweeps over each model random variable ("sinew" design).
     Use random starting points drawn from the joint density.
@@ -199,7 +138,7 @@ def eval_sinews(
         varname (str): Column name to give for sweep variable; default="sweep_var"
         indname (str): Column name to give for sweep index; default="sweep_ind"
         append (bool): Append results to conservative inputs?
-        skip (bool): Skip evaluation?
+        skip (bool): Skip evaluation of the functions?
 
     Returns:
         DataFrame: Results of evaluation or unevaluated design
@@ -336,7 +275,7 @@ def eval_hybrid(
     append=True,
     skip=False,
 ):
-    """Hybrid points for Sobol' indices
+    r"""Hybrid points for Sobol' indices
 
     Use the "hybrid point" design (Sobol', 1999) to support estimating Sobol'
     indices. Use gr.tran_sobol() to post-process the results and compute
@@ -351,7 +290,7 @@ def eval_hybrid(
             for nominal deterministic levels.
         varname (str): Column name to give for sweep variable; default="hybrid_var"
         append (bool): Append results to conservative inputs?
-        skip (bool): Skip evaluation?
+        skip (bool): Skip evaluation of the functions?
 
     Returns:
         DataFrame: Results of evaluation or unevaluated design
