@@ -1,20 +1,24 @@
 __all__ = [
-    "continuous_fit",
     "copy_meta",
     "custom_formatwarning",
     "df_equal",
     "df_make",
+    "marg_gkde",
+    "marg_named",
     "param_dist",
     "pipe",
-    "valid_dist"
+    "valid_dist",
 ]
 
+import grama as gr
 import pandas as pd
 import warnings
 
 from functools import wraps
 from numbers import Integral
 
+from scipy.stats import gaussian_kde
+
 from scipy.stats import alpha, anglit, arcsine, argus, beta, betaprime
 from scipy.stats import bradford, burr, burr12, cauchy, chi, chi2, cosine
 from scipy.stats import crystalball, dgamma, dweibull, erlang, expon, exponnorm
@@ -72,7 +76,7 @@ valid_dist = {
     "gamma": gamma,
     "gengamma": gengamma,
     "genhalflogistic": genhalflogistic,
-# "geninvgauss": geninvgauss,
+    # "geninvgauss": geninvgauss,
     "gilbrat": gilbrat,
     "gompertz": gompertz,
     "gumbel_r": gumbel_r,
@@ -99,7 +103,7 @@ valid_dist = {
     "loggamma": loggamma,
     "loglaplace": loglaplace,
     "lognorm": lognorm,
-# "loguniform": loguniform,
+    # "loguniform": loguniform,
     "lomax": lomax,
     "maxwell": maxwell,
     "mielke": mielke,
@@ -132,7 +136,7 @@ valid_dist = {
     "wald": wald,
     "weibull_min": weibull_min,
     "weibull_max": weibull_max,
-    "wrapcauchy": wrapcauchy
+    "wrapcauchy": wrapcauchy,
 }
 
 param_dist = {
@@ -244,18 +248,19 @@ def copy_meta(df_source, df_target):
     """Internal metadata copy tool
 
     Args:
-        df_source (DataFrame):
-        df_target (DataFrame):
+        df_source (DataFrame): Original dataframe
+        df_target (DataFrame): Target dataframe; receives metadata
 
     Returns:
         DataFrame: df_target with copied metadata
     """
-    df_target._grouped_by = getattr(df_source, '_grouped_by', None)
-    df_target._plot_info  = getattr(df_source, '_plot_info', None)
-    df_target._meta       = getattr(df_source, '_meta', None)
+    df_target._grouped_by = getattr(df_source, "_grouped_by", None)
+    df_target._plot_info = getattr(df_source, "_plot_info", None)
+    df_target._meta = getattr(df_source, "_meta", None)
 
     return df_target
 
+
 ## Pipe decorator
 class pipe(object):
     __name__ = "pipe"
@@ -291,12 +296,17 @@ class pipe(object):
         return result
 
     def __repr__(self):
-        return self.__name__ + ": " + self.function.__name__ + \
-            "\n Did you mean to use a full-prefix verb?"
+        return (
+            self.__name__
+            + ": "
+            + self.function.__name__
+            + "\n Did you mean to use a full-prefix verb?"
+        )
 
     def __call__(self, *args, **kwargs):
         return pipe(lambda x: self.function(x, *args, **kwargs))
 
+
 ## Safe length-checker
 def safelen(x):
     try:
@@ -304,9 +314,10 @@ def safelen(x):
     except TypeError:
         return 1
 
+
 ## DataFrame constructor utility
 def df_make(**kwargs):
-    """Construct a DataFrame
+    r"""Construct a DataFrame
 
     Helper function to construct a DataFrame.
 
@@ -356,8 +367,9 @@ def df_make(**kwargs):
 
     return df_res
 
+
 ## DataFrame equality checker
-def df_equal(df1, df2, close=False):
+def df_equal(df1, df2, close=False, precision=3):
     """Check DataFrame equality
 
     Check that two dataframes have the same columns and values. Allow column
@@ -381,7 +393,8 @@ def df_equal(df1, df2, close=False):
                 df1[df2.columns],
                 df2,
                 check_dtype=False,
-                check_exact=False
+                check_exact=False,
+                check_less_precise=precision,
             )
             return True
         except:
@@ -389,9 +402,10 @@ def df_equal(df1, df2, close=False):
     else:
         return df1[df2.columns].equals(df2)
 
+
 ## Fit a named scipy.stats distribution
-def continuous_fit(data, dist, name=True, sign=None):
-    """Fit scipy.stats continuous distirbution
+def marg_named(data, dist, name=True, sign=None):
+    r"""Fit scipy.stats continuous distirbution
 
     Fits a named scipy.stats continuous distribution. Intended to be used to
     define a marginal distribution from data.
@@ -409,30 +423,64 @@ def continuous_fit(data, dist, name=True, sign=None):
 
         >>> import grama as gr
         >>> from grama.data import df_stang
-        >>> param_E  = gr.continuous_fit(df_stang.E, "norm")
-        >>> param_mu = gr.continuous_fit(df_stang.mu, "beta")
-        >>> md = gr.Model("Marginal Example") >> \
-        >>>     gr.cp_marginals(E=param_E, mu=param_mu)
+        >>>     gr.cp_marginals(
+        >>>         E=gr.marg_named(df_stang.E, "norm"),
+        >>>         mu=gr.marg_named(df_stang.mu, "beta")
+        >>>     )
         >>> md.printpretty()
 
     """
     with warnings.catch_warnings():
         warnings.simplefilter("ignore")
-        res = valid_dist[dist].fit(data)
+        param = valid_dist[dist].fit(data)
 
-    res = dict(zip(param_dist[dist], res))
-
-    if name:
-        res["dist"] = dist
+    param = dict(zip(param_dist[dist], param))
 
     if sign is not None:
         if not (sign in [-1, 0, +1]):
             raise ValueError("Invalid `sign`")
-        res["sign"] = sign
+    else:
+        sign = 0
+
+    return gr.MarginalNamed(sign=sign, d_name=dist, d_param=param)
+
+
+## Fit a gaussian kernel density estimate (KDE) to data
+def marg_gkde(data, sign=None):
+    r"""Fit a gaussian KDE to data
+
+    Fits a gaussian kernel density estimate (KDE) to data.
+
+    Args:
+        data (iterable): Data for fit
+        sign (bool): Include sign? (Optional)
+
+    Returns:
+        gr.MarginalGKDE: Marginal distribution
+
+    Examples:
+
+        >>> import grama as gr
+        >>> from grama.data import df_stang
+        >>> md = gr.Model("Marginal Example") >> \
+        >>>     gr.cp_marginals(
+        >>>         E=gr.marg_gkde(df_stang.E),
+        >>>         mu=gr.marg_gkde(df_stang.mu)
+        >>>     )
+        >>> md.printpretty()
+
+    """
+    kde = gaussian_kde(data)
+    if sign is not None:
+        if not (sign in [-1, 0, +1]):
+            raise ValueError("Invalid `sign`")
+    else:
+        sign = 0
+
+    return gr.MarginalGKDE(kde, sign=sign)
 
-    return res
 
 ## Monkey-patched warning fcn
 def custom_formatwarning(msg, *args, **kwargs):
     # ignore everything except the message
-    return "Warning: " + str(msg) + '\n'
+    return "Warning: " + str(msg) + "\n"
